/* tsxplot.c                      tab  May 2014 */

/* Plot TSPEC profiles and spectra using PGPLOT. */

#include "/usr/local/include/fitsio.h"
#include "tsx.h"
#include "mua.h"
#include "cfua.h"
#include "galactic.h"
#include "cpg-util.h"
#include "cpgplot.h"




/*----------------------------------------------------------------------
 Command line syntax.
*/
void tsxplot_syntax()
{

printf("Syntax: tsxplot (raw TripleSpec FITS file or root name of summed file)\n");
printf("\n");
printf("  Create PGPLOT PostScript plots.\n");
printf("Options\n");
printf("  x=      : name of column to plot on x axis (default=wave/arcsec).\n");
printf("  y=      : name of column to plot on y axis (default=sum/flux_mb).\n");
printf("  z=      : name of 2nd column to plot on y axis (default=none).\n");
printf("  object= : print this string in title of plot.\n");
printf("  -sp     : read -sp#.tbl files.\n");
printf("  -pro    : read -pro#.tbl files.\n");
printf("  -ps     : create PS plot file '(root).ps' \n");
printf("  ps=     : specify filename of PostScript plot file.\n");
printf("    eo=   : Echelle Order to plot (3,4,5,6,7) (default: show all orders).\n");
printf("  xmin=   : Minimum X value to plot.\n");
printf("  xmax=   : Maximum X value to plot.\n");
printf("  ymin=   : Minimum Y value to plot.\n");
printf("  ymax=   : Maximum Y value to plot.\n");
printf("   bin=   : Number of pixels to bin (average) (default: no binning).\n");
printf("\n");

return;
}

/* ----------------------------------------------------------------------
 Append a slash if needed.
*/
void tsxplot_append_slash( char wrd[] )
{
int ii;
ii = clc(wrd);
if (ii >= 0) { if (wrd[ii] != '/') { wrd[(ii+1)]='/';   wrd[(ii+2)]='\0'; } }
return;
}



/* ----------------------------------------------------------------------
  Find range of a column.
*/
void tsxplot_findcol( char hdr[], char name[], int *i1, int *i2 )
{
/**/
int ii,ii1,ii2,kk;
/**/
ii = cindex(hdr,name);
if (ii < 0) { printf("***error:1: finding column '%s'.\n",name); exit(1); }
ii1=ii;
while ((ii1 > -1)&&(hdr[ii1] != '|')) { --ii1; }
if (ii1 < 0) { printf("***error:2: finding column '%s'.\n",name); exit(1); }
ii2=ii; kk=clc(hdr);
while ((ii2 <= kk)&&(hdr[ii2] != '|')) { ++ii2; }
if (ii2 > kk) { printf("***error:3: finding column '%s'.\n",name); exit(1); }
*i1 = ii1;
*i2 = ii2;
return;
}




/* - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   Plot TSPEC profiles and spectra using PGPLOT.
*/
int main(int argc, char *argv[])
{
/**/
char arg[202][200];
char wrd[300];
char root[200];
char line[500];
char infile[200];
char tsxout[200];
char device[100];
char object[100];
char hdr[200];
char xcol[30];
char ycol[30];
char zcol[30];
char psroot[300];
/**/
const int mm=3000;
float arr[mm],xx[mm],yy[mm],zz[mm];
float sch,range,median,xmin,xmax,ymin,ymax;
float xs,xe,ys,ye;
/**/
FILE *infu;
/**/
int eo,kk,ee,psflag,narg,ii,nn,nh,nv,plotpro,plotsp;
int xi1,xi2,yi1,yi2,zi1,zi2,bin;
/**/

/* Copy arguments. */
cargcopy(argv, argc, &narg, arg);

/* Syntax. */
if (narg == 0) { tsxplot_syntax(); exit(0); }

/* Options. */
psflag  = cfindarg(arg,&narg,"-ps" ,'s',wrd);
plotpro = cfindarg(arg,&narg,"-pro" ,'s',wrd);
plotsp  = cfindarg(arg,&narg,"-sp"  ,'s',wrd);

strcpy(xcol,"none"); if (cfindarg(arg,&narg,"x=",':',wrd)) { strcpy(xcol,wrd); }
strcpy(ycol,"none"); if (cfindarg(arg,&narg,"y=",':',wrd)) { strcpy(ycol,wrd); }
strcpy(zcol,"none"); if (cfindarg(arg,&narg,"z=",':',wrd)) { strcpy(zcol,wrd); }

strcpy(psroot,""); if (cfindarg(arg,&narg,"ps=",':',wrd)) { strcpy(psroot,wrd); psflag=1; }

eo=0; if (cfindarg(arg,&narg,"eo=",':',wrd)) { eo = GLV(wrd,1); }

bin=0; if (cfindarg(arg,&narg,"bin=",':',wrd)) { bin = GLV(wrd,1); }

xs=-1.e+30; if (cfindarg(arg,&narg,"xmin=",':',wrd)) { xs = GLV(wrd,1); }
xe= 1.e+30; if (cfindarg(arg,&narg,"xmax=",':',wrd)) { xe = GLV(wrd,1); }
ys=-1.e+30; if (cfindarg(arg,&narg,"ymin=",':',wrd)) { ys = GLV(wrd,1); }
ye= 1.e+30; if (cfindarg(arg,&narg,"ymax=",':',wrd)) { ye = GLV(wrd,1); }

strcpy(object,"");
if (cfindarg(arg,&narg,"object="  ,':',wrd)) { strcpy(object,wrd); }

/* Root. */
strcpy(root,arg[1]);
ii=cindex(root,".fits"); if (ii > -1) root[ii]='\0';
printf("root=%s\n",root);

/* Check TSXOUT environment variable. */
sprintf(wrd,"=%s=",getenv("TSXOUT"));
if (strcmp(wrd,"=(null)=") == 0) {
  strcpy(tsxout,"");
} else {
  substrcpy_terminate(wrd,1,clc(wrd)-1,tsxout,0);
  if (FileExist(tsxout) == 0) {
    printf("***error: Cannot find TSXOUT directory '%s'.\n",tsxout); exit(1);
  }
  tsxplot_append_slash(tsxout);
}


/* . . . profiles. . . . */                 /* sprintf(device,"%s-pro.png/PNG",root); */
if (plotpro) {

if (strcmp(xcol,"none") == 0) { strcpy(xcol,"arcsec"); }
if (strcmp(ycol,"none") == 0) { strcpy(ycol,"flux_mb"); }
printf("Plot profile: '%s' vs. '%s'.\n",ycol,xcol);

/* Begin plot. */
strcpy(device,"/XWINDOW");
if (psflag) { 
  if (strcmp(psroot,"") == 0) {
    sprintf(device,"%s%s-pro.ps/CPS",tsxout,root);
  } else {
    sprintf(device,"%s%s-pro.ps/CPS",tsxout,psroot);
  }
  printf("Writing '%s'.\n",device);
}
if (eo == 0) { nh=2; nv=3; sch=2.0; }
        else { nh=1; nv=1; sch=2.0; }
cpgbeg(0,device,nh,nv);
cpgsvp(0.10,0.90,0.15,0.85);
cpgsch(sch);
cpgslw(1.0);
cpgsls(1);

/* Each echelle order. */
for (ee=3; ee<8; ++ee) {
if ((eo == 0)||(ee == eo)) {

/* Read data. */
  sprintf(infile,"%s-pro%d.tbl",root,ee);
  infu = fopen_read(infile);
  fgetline(hdr,infu);
  tsxplot_findcol(hdr,xcol,&xi1,&xi2);
  tsxplot_findcol(hdr,ycol,&yi1,&yi2);
  nn=0;
  xmin= 1.e+30; xmax=-1.e+30;
  ymin= 1.e+30; ymax=-1.e+30;
  while (fgetline(line,infu)) { if (line[0] != '|') {
    xx[nn] = cvalread0(line,xi1,xi2);
    if ((xx[nn] > xs)&&(xx[nn] < xe)) {
      yy[nn] = cvalread0(line,yi1,yi2);
      if (strcmp(ycol,"flux_mb") == 0) {
        zz[nn] = GLV(line,11);     /* bfit_mb */
      } else { zz[nn]=0.; }
      ++nn;
      if (nn > mm-3) { printf("***error: too many points.\n"); exit(1); }
    }
  }}
  fclose(infu);
  if (bin > 1) {
    kk=nn; cDoBin4(bin,&kk,xx,0);
    kk=nn; cDoBin4(bin,&kk,yy,0);
    kk=nn; cDoBin4(bin,&kk,zz,0);
    nn=kk;
  }
  for (ii=0; ii<nn; ++ii) {
    if (xx[ii] < xmin) xmin=xx[ii];
    if (xx[ii] > xmax) xmax=xx[ii];
    if (yy[ii] < ymin) ymin=yy[ii];
    if (yy[ii] > ymax) ymax=yy[ii];
  }

/* X range. */
  range = xmax - xmin;
  xmin = xmin - (range * 0.02);
  xmax = xmax + (range * 0.02);
  
/* Find median, set y range. */
  for (ii=0; ii<nn; ++ii) { arr[ii]=yy[ii]; }
  median = cfind_median(nn,arr);
  range = ymax - median;
  ymin = median - (range * 0.20);
  ymax = ymax   + (range * 0.05);
/* override. */
  if (ys > -1.e+29) { ymin = ys; }
  if (ye <  1.e+29) { ymax = ye; }

/* Plot. */
  cpgpage();
  cpgswin(xmin, xmax, ymin, ymax);
  cpgbox("BCNST",0.,0,"BCNST",0.,0);
  cpgline(nn,xx,yy);
  cpgsci(2); cpgsls(4);
  cpgline(nn,xx,zz);
  cpgsci(1); cpgsls(1);
  if (ee == 3) { sprintf(wrd,"%s : %s : 3 (K band)",object,root);   } else {
  if (ee == 4) { sprintf(wrd,"%s : %s : 4 (H band)",object,root);   } else {
  if (ee == 5) { sprintf(wrd,"%s : %s : 5 (J band)",object,root);   } else {
  if (ee == 6) { sprintf(wrd,"%s : %s : 6 (z/J band)",object,root); } else {
  if (ee == 7) { sprintf(wrd,"%s : %s : 7 (z band)",object,root);   }      }}}}
  cpgmtxt("T",1.0,0.5,0.5,wrd);
  cpgmtxt("L",1.7,0.5,0.5,ycol);
  cpgmtxt("B",2.5,0.5,0.5,xcol);

}}
cpgend(); /* End plot. */

}


/* . . . spectra. . . . */
if (plotsp) {

if (strcmp(xcol,"none") == 0) { strcpy(xcol,"wave"); }
if (strcmp(ycol,"none") == 0) { strcpy(ycol,"sum");  }
if (strcmp(zcol,"none") == 0) { strcpy(zcol,"error");}
printf("Plot spectrum: '%s' vs. '%s' (and '%s').\n",ycol,xcol,zcol);

/* Begin plot. */
strcpy(device,"/XWINDOW");
if (psflag) { 
  if (strcmp(psroot,"") == 0) {
    sprintf(device,"%s%s-sp.ps/CPS",tsxout,root);
  } else {
    sprintf(device,"%s%s-sp.ps/CPS",tsxout,psroot);
  }
  printf("Writing '%s'.\n",device);
}
if (eo == 0) { nh=2; nv=3; sch=2.0; }
        else { nh=1; nv=1; sch=2.0; }
cpgbeg(0,device,nh,nv);
cpgsvp(0.10,0.90,0.15,0.85);
cpgsch(sch);
cpgslw(1.0);
cpgsls(1);

/* Each echelle order. */
for (ee=3; ee<8; ++ee) {
if ((eo == 0)||(ee == eo)) {

/* Read data. */
  sprintf(infile,"%s-sp%d.tbl",root,ee);
  infu = fopen_read(infile);
  fgetline(hdr,infu);
  xi1=0; xi2=0;  yi1=0; yi2=0;  zi1=0; zi2=0;
  tsxplot_findcol(hdr,xcol,&xi1,&xi2);
  tsxplot_findcol(hdr,ycol,&yi1,&yi2);
  if (cindex(hdr,zcol) > -1) {
    tsxplot_findcol(hdr,zcol,&zi1,&zi2);
  }
  nn=0;
  xmin= 1.e+30; xmax=-1.e+30;
  ymin= 1.e+30; ymax=-1.e+30;
  while (fgetline(line,infu)) { if (line[0] != '|') {
    xx[nn] = cvalread0(line,xi1,xi2);
    if ((xx[nn] > xs)&&(xx[nn] < xe)) {
      yy[nn] = cvalread0(line,yi1,yi2);
      if (zi2 > 0) {
        zz[nn] = cvalread0(line,zi1,zi2);
      } else { zz[nn]=0.; }
      ++nn;
      if (nn > mm-3) { printf("***error: too many points.\n"); exit(1); }
    }
  }}
  fclose(infu);
  if (bin > 1) {
    kk=nn; cDoBin4(bin,&kk,xx,0);
    kk=nn; cDoBin4(bin,&kk,yy,0);
    if (cindex(zcol,"error") > -1) { kk=nn; cDoBin4(bin,&kk,zz,1); }
                              else { kk=nn; cDoBin4(bin,&kk,zz,0); }
    nn=kk;
  }
  for (ii=0; ii<nn; ++ii) {
    if (xx[ii] < xmin) xmin=xx[ii];
    if (xx[ii] > xmax) xmax=xx[ii];
    if (yy[ii] < ymin) ymin=yy[ii];
    if (yy[ii] > ymax) ymax=yy[ii];
  }

/* X range. */
  range = xmax - xmin;
  xmin = xmin - (range * 0.02);
  xmax = xmax + (range * 0.02);

/* Find median, set y range. */
  for (ii=0; ii<nn; ++ii) { arr[ii]=yy[ii]; }
  median = cfind_median(nn,arr);
  kk = cnint((float)nn * 0.98);
  range = arr[kk];
  ymin = 0.   - (range * 0.10);
  ymax = range * 1.50;
/* override. */
  if (ys > -1.e+29) { ymin = ys; }
  if (ye <  1.e+29) { ymax = ye; }
/* check. */
  if ((ymax-ymin) < 1.e-29) { ymax = ymin + 1.e-28; }

/* Plot. */
  cpgpage();
  cpgswin(xmin, xmax, ymin, ymax);
  cpgbox("BCNST",0.,0,"BCNST",0.,0);
  cpgline(nn,xx,yy);
  cpgsci(2);
  cpgline(nn,xx,zz);
  cpgsci(1);
  if (ee == 3) { sprintf(wrd,"%s : %s : 3 (K band)",object,root);   } else {
  if (ee == 4) { sprintf(wrd,"%s : %s : 4 (H band)",object,root);   } else {
  if (ee == 5) { sprintf(wrd,"%s : %s : 5 (J band)",object,root);   } else {
  if (ee == 6) { sprintf(wrd,"%s : %s : 6 (z/J band)",object,root); } else {
  if (ee == 7) { sprintf(wrd,"%s : %s : 7 (z band)",object,root);   }      }}}}
  cpgmtxt("T",1.0,0.5,0.5,wrd);
  cpgmtxt("L",1.7,0.5,0.5,ycol);
  cpgmtxt("B",2.5,0.5,0.5,xcol);
  cpgsci(2);
  cpgmtxt("R",1.2,0.5,0.5,"Error");
  cpgsci(1);
  for (ii=0; ii<nn; ++ii) { zz[ii]=0.; }
  cpgsls(2); cpgline(nn,xx,zz); cpgsls(1);

}}
cpgend(); /* End plot. */

}

return(0);
}
